@@ -7,7 +7,7 @@
  *  Typedefs, other constants, etc. begin with "DDCA_".
  */
 
-// Copyright (C) 2014-2023 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2014-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #ifndef DDCUTIL_C_API_H_
@@ -178,6 +178,9 @@ ddca_rc_desc(
 //
 
 /** Performs library initialization.
+ *
+ *  @deprecated
+ *  Use ddca_init2()
  *
  *  @param[in]  library_options  string of **libddcutil** options
  *  @param[in]  syslog_level     severity cutoff for system log
@@ -200,6 +203,35 @@ ddca_init(const char *      libopts,
           DDCA_Init_Options opts);
 
 
+/** Performs library initialization.
+ *
+ *  @param[in]  library_options  string of **libddcutil** options
+ *  @param[in]  syslog_level     severity cutoff for system log
+ *  @param[in]  opts             option flags
+ *  @param[out] infomsg_loc      if non-null, return null terminated list of
+ *                               informational msgs here
+ *  @return status code
+ *
+ * Unless flag DDC_INIT_OPTIONS_DISABLE_CONFIG_FILE is set in **opts**,
+ * libddcutil options are read from the ddcutil configuration file.
+ * These are combined with any options passed in string **libopts**
+ * and then processed.
+ *
+ * If the returned status code is other than **DDCRC_OK**, a detailed
+ * error report can be obtained using #ddca_get_error_detail().
+ *
+ * The caller is responsible for freeing the null terminated list of
+ * messages returned in infomsg_loc.
+ *
+ * @since 2.1.0
+ */
+DDCA_Status
+ddca_init2(const char *     libopts,
+          DDCA_Syslog_Level syslog_level_arg,
+          DDCA_Init_Options opts,
+          char***           infomsg_loc);
+
+
 //
 // Global Settings
 //
@@ -229,30 +261,90 @@ ddca_is_verify_enabled(void);
 // Performance
 //
 
-/** Sets the sleep multiplier factor for the open display on current thread.
+
+/** Sets an explicit sleep multiplier factor for the current thread.
+ *  If set, it takes precedence over any other sleep multiplier calculation.
  *
- *  The semantics of this function has changed. Prior to release 1.5,
- *  this function set the sleep multiplier for the current thread.
- *  As of release 1.5, it sets the sleep multiplier for open display
- *  (if any) on the current thread.
+ *  @deprecated
+ *  This function provides backwards compatibility with applications
+ *  written for libddcutil release 1.x.  Its use in new applications
+ *  is discouraged.
  *
  *  @param[in]  multiplier, must be >= 0 and <= 10
- *  @return     old multiplier, -1.0f if invalid multiplier specified, or no display open
+ *  @return     old multiplier, -1.0f if invalid multiplier specified
+ *              or multiplier was not previously set
  */
-double
+__attribute__ ((deprecated)) double
 ddca_set_sleep_multiplier(double multiplier);
 
-/** Gets the sleep multiplier for the open display on the current thread
+
+/** Gets the explicit sleep multiplier for the current thread.
+ *
+ *  @deprecated
+ *  This function provides backwards compatibility with applications
+ *  written for libddcutil release 1.x.  It's use in new applications
+ *  is discouraged.
  *
- *  As of release 1.5, the semantics of this function has changed.
  *  See #ddca_set_sleep_multiplier().
  *
- *  @return  sleep multiplier, -1.0f if no display open on current thread
+ *  @return  sleep multiplier, -1.0f if not set
  */
-double
+__attribute__ ((deprecated)) double
 ddca_get_sleep_multiplier();
 
 
+/** Sets an explicit sleep multiplier factor for the specified display.
+ *  If set, it takes precedence over any other sleep multiplier calculation,
+ *  except for (deprecated) thread sleep multiplier.
+ *
+ *  @param[in]  dref        display reference
+ *  @param[in]  multiplier  must be >= 0 and <= 10
+ *  @retval     DDCRC_OK
+ *  @retval     DDCRC_ARG   invalid display reference or multiplier value
+ */
+DDCA_Status
+ddca_set_display_sleep_multiplier(
+      DDCA_Display_Ref       dref,
+      DDCA_Sleep_Multiplier  multiplier);
+
+
+/** Gets the current effective sleep multiplier for the specified display.
+ *
+ *  This value can vary if dynamic sleep adjustment is active.
+ *
+ *  @param   dref           display reference
+ *  @param   multiplier_loc where to return answer
+ *  @retval  DDCRC_OK
+ *  @retval  DDCRC_ARG      invalid display reference
+ */
+DDCA_Status
+ddca_get_current_display_sleep_multiplier(
+      DDCA_Display_Ref        dref,
+      DDCA_Sleep_Multiplier*  multiplier_loc);
+
+
+/** Controls whether dynamic sleep adjustment is enabled.
+ *  This is a global setting that applies to all displays.
+ *
+ *  @param  onoff
+ *  @return previous setting
+ *
+ *  @since 2.1.0
+ */
+bool
+ddca_enable_dynamic_sleep(bool onoff);
+
+
+/** Reports whether dynamic sleep adjustment is enabled.
+ *
+ *  @return current setting
+ *
+ *  @since 2.1.0
+ */
+bool
+ddca_is_dynamic_sleep_enabled();
+
+
 //
 // Output Redirection
 //
@@ -663,6 +755,24 @@ ddca_get_display_ref(
       DDCA_Display_Identifier did,
       DDCA_Display_Ref*       dref_loc);
 
+/** Checks whether a #DDCA_Display_Ref is valid
+ *
+ *  @param   dref                  display reference to validate
+ *  @param   require_not_asleep    include check for DPMS sleep state
+ *  @retval  DDCRC_OK
+ *  @retval  DDCRC_ARG             dref == NULL or does not point to a valid DDCA_Display_Ref
+ *  @retval  DDCRC_INTERNAL_ERROR  dref->drm_connector == NULL
+ *  @retval  DDCRC_DISCONNECTED    monitor has been disconnected
+ *  @retval  DDCRC_DPMS_ASLEEP     monitor in a DPMS sleep state
+ *  @retval  DDCRC_INVALID_DISPLAY not found
+ *
+ *  @since 2.1.0
+ */
+DDCA_Status
+ddca_validate_display_ref(
+      DDCA_Display_Ref  dref,
+      bool              require_not_asleep);
+
 /** Returns a string representation of a display reference
  *
  *  The returned value is valid until the next call to this function on
@@ -856,7 +966,7 @@ ddca_report_parsed_capabilities(
       int                      depth);
 
 /** Returns the VCP feature codes defined in a
- *  parsed capabilities record as a #DDCA_Feature_LIst
+ *  parsed capabilities record as a #DDCA_Feature_List
  *
  *  @param[in] parsed_caps  parsed capabilities
  *  @return bitfield of feature ids
@@ -1027,7 +1137,7 @@ ddca_get_feature_metadata_by_dh(
       bool                        create_default_if_not_found,
       DDCA_Feature_Metadata **    meta_loc);
 
-/** Frees a #DDCA_Feature_Metadata instance.
+/** Frees a #DDCA_Feature_Metadata instance
  *
  *  @param[in] metadata pointer to instance
  *  @retval   0  normal
@@ -1035,6 +1145,8 @@ ddca_get_feature_metadata_by_dh(
  *
  *  @remark
  *  It is not an error if the ***metadata*** pointer argument is NULL
+ *  @remark
+ *  This is a convenience function.
  */
 void
 ddca_free_feature_metadata(DDCA_Feature_Metadata * metadata);
@@ -1545,7 +1657,8 @@ ddca_set_any_vcp_value(
  *  calibrated and restored when the calibration is applied.
  *
  *  @param[in]  ddca_dh                   display handle
- *  @param[out] profile_values_string_loc address at which to return string
+ *  @param[out] profile_values_string_loc address at which to return string,
+ *                                        caller is responsible for freeing
  *  @return     status code
  */
 DDCA_Status
@@ -1564,8 +1677,8 @@ ddca_get_profile_related_values(
  *  @remark
  *  If **ddca_dh** is NULL, this function opens the first display
  *  that matches the display identifiers in the **profile_values_string**.
- *  If **ddca_dh** is non-NULL, then the identifiers in **profile_values_string**
- *  must be consistent with the open display.
+ *  If **ddca_dh** is non-NULL, then the identifiers in
+ *  **profile_values_string**  must be consistent with the open display.
  *  @remark
  *  The non-NULL case exists to handle the unusual situation where multiple
  *  displays have the same manufacturer, model, and serial number,
@@ -1580,6 +1693,108 @@ ddca_set_profile_related_values(
       char *               profile_values_string);
 
 
+//
+//  Report display status changes
+//
+
+/** Registers a function to be called called when a change in display status is
+ *  detected. It is not an error if the function is already registered.
+ *
+ *  @param[in] func   function of type #DDCA_Display_Detection_Callback_Func()
+ *  @return    DDCRC_OK
+ *  @retval    DDCRC_INVALID_OPERATION ddcutil not built with UDEV support,
+ *                                     or not all video devices support DRM
+ *
+ *  @since 2.1.0
+ */
+DDCA_Status
+ddca_register_display_status_callback(DDCA_Display_Status_Callback_Func func);
+
+/** Removes a function from the list of registered callbacks
+ *
+ *  @param[in] func            function that has already been registered
+ *  @retval    DDCRC_OK        function removed from list
+ *  @retval    DDCRC_INVALID_OPERATION ddcutil not built with UDEV support,
+ *                                     or not all video devices support DRM
+ *  @retval    DDCRC_NOT_FOUND function not registered
+ *
+ *  @since 2.1.0
+ */
+DDCA_Status
+ddca_unregister_display_status_callback(DDCA_Display_Status_Callback_Func func);
+
+/** Returns the name of a #DDCA_Display_Event_Class
+ *
+ *  @param  event_class event class id
+ *  @return             printable name
+ *
+ *  @remark
+ *  The value returned exists in an internal ddcutil table.
+ *  Caller should not free.
+ *
+ *  @since 2.1.0
+ */
+const char *
+   ddca_display_event_class_name(DDCA_Display_Event_Class event_class);
+
+/** Returns the name of a #DDCA_Display_Event_Type
+ *
+ *  @param  event_type  event type id
+ *  @return             printable event type name
+ *
+ *  @remark
+ *  The value returned exists in an internal ddcutil table.
+ *  Caller should not free.
+ *
+ *  @since 2.1.0
+ */
+const char *
+   ddca_display_event_type_name(DDCA_Display_Event_Type event_type);
+
+/** Start the thread watching for display status changes.
+ *
+ *  @remark
+ *  The thread can also be started using option --enable-watch_displays.
+ *
+ *  @param  enabled_clases  event classes to watch
+ *  @retval DDCRC_OK
+ *  #retval DDCRC_ARG     no event classes specified
+ *  @retval DDCRC_INVALID_OPERATION watch thread already running
+ *
+ *  @since 2.1.0
+ */
+DDCA_Status
+ddca_start_watch_displays(DDCA_Display_Event_Class enabled_classes);
+
+/** Terminate the thread that watches for display status changes.
+ *
+ *  This function is a hack.  Without it, the thread can
+ *  continue running even though the application has exited.
+ *
+ *  @param wait  Wait for watch thread to actually terminate
+ *  @retval DDCRC_OK
+ *  @retval DDCRC_INVALID_OPERATION  watch thread not executing
+ *
+ *  If this function is being called as part of termination
+ *  by the client, there's no need to wait for the watch thread
+ *  to actually finish.
+ *
+ *  @since 2.1.0
+ */
+DDCA_Status
+ddca_stop_watch_displays(bool wait);
+
+/** If the watch thread is currently executing returns, reports the
+ *  currently active display event classes as a bit flag.
+ *
+ *  @param  classes_loc  where to return bit flag
+ *  @retval DDCRC_OK
+ *  @retval DDCRC_INVALID_OPERATION watch thread not executing
+ */
+DDCA_Status
+ddca_get_active_watch_classes(DDCA_Display_Event_Class * classes_loc);
+
+
 
 #ifdef __cplusplus
 }
